/****************************************************************************
 *
 *      $Id: chipset.h,v 1.1 2003/08/06 22:53:46 benjl Exp $
 *
 *      Description: This file describes the interface to a generic chipset
 *      driver.
 *
 *      Author:      Ben Leslie
 *      Created:     Wed Aug  6 17:05:45 2003
 *
 ****************************************************************************/

#include <types.h>
#include <stdint.h>

/* This structure defines the operations table */

struct chipset_ops {
	/* Read and PCI configuration registers */
	void (*conf_write_8) (void *pvt, int bus, int device, int function,
			      int register_num, uint8_t value);
	void (*conf_write_16) (void *pvt, int bus, int device, int function,
			      int register_num, uint16_t value);
	void (*conf_write_32) (void *pvt, int bus, int device, int function,
			      int register_num, uint32_t value);
	uint8_t (*conf_read_8) (void *pvt, int bus, int device, int function,
				 int register_num);
	uint16_t (*conf_read_16) (void *pvt, int bus, int device, int function,
				 int register_num);
	uint32_t (*conf_read_32) (void *pvt, int bus, int device, int function,
				 int register_num);
	/* Map a section of PCI IO space */
	bus_space_t (*pci_iospace_map) (void *pvt, bus_addr_t, uintptr_t size,
					int flags);
	/* Register a PCI interrupt */
	int (*pci_register_interrupt) (void *pvt, int irq, exec_handle_t handler);
};

/* The chipset structure has a pointer to the ops table, and some private 
   data */

struct chipset {
	struct chipset_ops *ops;
	void *pvt;
};
