/*
  L4 Environment - Ben Leslie 2003

  The L4 Environment a more convenviant set of abstractions
  than the L4::Pistachio conveniance interface.

  This environment is not meant as a general purpose L4 library;
  it actually makes some policy decesions, which are not always 
  going to be the best policy, but it certainly encapsulates a set
  of commonly used policies to make implementing a new OS personality
  a fair bit easier, especially in undergraduate projects.

  More of these policies are based on experience using the L4v2 API.

*/

#include <l4.h>

extern L4_ThreadId_t root_thread;

int l4e_init(void);

/* Output functions */
void l4e_print_string(char *s);
void l4e_print_error(L4_Word_t ec);
void l4e_print_fpage(L4_Fpage_t fpage);
int l4e_printf(const char *fmt, ...);

/* TID allocation functions */
L4_ThreadId_t l4e_get_new_tid (void);
L4_ThreadId_t l4e_task_2_tid(int taskid);
int l4e_tid_2_task(L4_ThreadId_t tid);

/* Thread and task handling function */
int l4e_start_thread(L4_ThreadId_t tid, void *entrypoint, void *stack);
int l4e_task_new(L4_ThreadId_t task, L4_ThreadId_t pager, 
		 void *entrypoint, void *stack);

/* Find valid memory */
void l4e_find_memory(L4_Word_t *low, L4_Word_t *high);

/* Request a page from sigma0 */
int l4e_request_page(void *physaddr, int cached);

/* Flush L4 Caches */
void l4e_cache_flush(void);
