#include <types.h>
#include <stddef.h>
#include <assert.h>
#include "dev.h"
#include <l4e.h>
#include <l4.h>

extern void *frame_alloc(void);

#define IRQS 8

struct irq_handlers {
	void (*function) (void *);
	void *data;
};

static struct irq_handlers irq_handlers[IRQS];

mem_handle_t
map_physical(host_handle_t host, void *phys_addr, uintptr_t size)
{
	l4e_request_page(phys_addr, 0);

	return phys_addr;
}

void
irq_handler(L4_ThreadId_t tid, L4_MsgTag_t tag, L4_Msg_t *msg)
{
	int irq = tid.global.X.thread_no;
	irq_handlers[irq].function(irq_handlers[irq].data);
}

int
register_cpu_interrupt_handler(host_handle_t host, int irq, 
			       void *function, void *data)
{
	L4_ThreadId_t tid;
	int res;

	tid.global.X.thread_no = irq;
	tid.global.X.version = 1;

	res = L4_ThreadControl(tid, tid, L4_nilthread, root_thread, (void*) -1);
	/* EVIL EVIL EVIL - shouldn't have L4_Pager in above code */

	if (res != 1) {
		return 1;
	}

	irq_handlers[irq].function = function;
	irq_handlers[irq].data = data;

	return 0;
}

void
dev_sleep(int micros)
{
	L4_Sleep(L4_TimePeriod(micros));
}

void
dev_yield(void)
{
	L4_ThreadSwitch(L4_anythread);
}

int
get_pinnable_memory(uintptr_t size, mem_handle_t *handle)
{
	assert(size < PAGESIZE); /* We place this restriction to keep
				    things simple */
	*handle = frame_alloc();
	return 0;
}

dma_addr_t
setup_dma(dma_handle_t dma, mem_handle_t handle, size_t size, 
	  direction_t dir)
{
	return (dma_addr_t) handle;
}

